from iso3166 import countries as co
from pandas import DataFrame, concat, read_csv
from csv import QUOTE_NONNUMERIC
from constants import ADMIN_PW


def country_conversion(political_unit) -> str:
    codes = co.get(political_unit)
    return codes.name


def select_columns() -> DataFrame:
    min_year = 2010
    fields = [
        "POLITICAL_UNIT",
        "NAME",
        "WGMS_ID",
        "YEAR",
        "AREA_SURVEY_YEAR",
        "AREA_CHANGE",
        "THICKNESS_CHG",
        "VOLUME_CHANGE",
    ]
    iter_csv = read_csv(
        "../data/WGMS-FoG-2019-12-D-CHANGE.csv",
        skipinitialspace=True,
        usecols=fields,
        iterator=True,
        chunksize=100,
        converters={"POLITICAL_UNIT": country_conversion},
    )
    data = concat([chunk[chunk["YEAR"] > min_year] for chunk in iter_csv])
    return data


def create_databases(df):
    users = {"UID": [7843], "USERNAME": ["admin"], "PASSWORD": [ADMIN_PW]}
    files = {
        "glacier": "../data/glacier.csv",
        "annual_data": "../data/annual_data.csv",
        "annual_change": "../data/annual_change.csv",
        "users": "../data/users.csv",
    }
    dataframes = {
        "glacier": df[["POLITICAL_UNIT", "NAME", "WGMS_ID"]].drop_duplicates(),
        "annual_data": df[["WGMS_ID", "YEAR", "AREA_SURVEY_YEAR"]],
        "annual_change": df[
            ["WGMS_ID", "YEAR", "AREA_CHANGE", "THICKNESS_CHG", "VOLUME_CHANGE"]
        ],
        "users": DataFrame(users),
    }
    for key, val in dataframes.items():
        val.to_csv(files[key], index=False, quoting=QUOTE_NONNUMERIC)


def main():
    df = select_columns()
    create_databases(df)


if __name__ == "__main__":
    main()
